#pragma once

#include <stdint.h>
#include <avr/pgmspace.h>

namespace Devices
{
    extern uint8_t const GRAPHICAL_TO_CHARACTER_FONT[1024] PROGMEM;

    template <class DriverT>
    class GraphicalToCharacter
    {
        private:

            static uint8_t x;
            static uint8_t y;
            
        public:

            static void Init()
            {
                x = 0;
                y = 0;
                DriverT::Init();
            }

            static void Move(uint8_t const x_, uint8_t const y_)
            {
                x = x_;
                y = y_;
            }

            static void PutChar(char const ch)
            {
                uint8_t index = ch & 0x7f;
                uint8_t buffer[8];

                for (int i = 0; i < 8; ++i)
                {
                    buffer[i] = pgm_read_byte(GRAPHICAL_TO_CHARACTER_FONT + index * 8 + i);
                }

                DriverT::Draw(x * 8, y, sizeof(buffer), buffer);
                
                ++x;

                if (x >= 16)
                {
                    x = 0;
                    ++y;
                }

                if (y >= 8)
                {
                    y = 0;
                }
            }

            static void PutString(char const * const s)
            {
                char const * it = s;

                while (*it) 
                {
                    PutChar(*it);
                    ++it;
                }
            }

            static void Clear()
            {
                DriverT::Clear();
                x = 0;
                y = 0;
            }
    };

    template <class DriverT> uint8_t GraphicalToCharacter<DriverT>::x = 0;
    template <class DriverT> uint8_t GraphicalToCharacter<DriverT>::y = 0;
}