#pragma once

#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "port.hpp"
#include "external_interrupt.hpp"
#include "timer0.hpp"
#include "timer2.hpp"
#include "power_management.hpp"
#include "usart_spi.hpp"
#include "adc.hpp"
#include "watchdog.hpp"


namespace Devices::Mcu
{
    class Atmega328
    {
        public:

            constexpr static unsigned long const ClockFrequency = F_CPU;

#ifdef PORTA
        private:

            struct PortARegisters
            {
                constexpr static volatile uint8_t * const PORT() { return &PORTA; }
                constexpr static volatile uint8_t * const DDR() { return &DDRA; }
                constexpr static volatile uint8_t * const PIN() { return &PINA; }
            };

        public:

            using PortA = Port<PortARegisters>;
#endif

#ifdef PORTB
        private:

            struct PortBRegisters
            {
                constexpr static volatile uint8_t * const PORT() { return &PORTB; }
                constexpr static volatile uint8_t * const DDR() { return &DDRB; }
                constexpr static volatile uint8_t * const PIN() { return &PINB; }
            };

        public:

            using PortB = Port<PortBRegisters>;
#endif

#ifdef PORTC
        private:

            struct PortCRegisters
            {
                constexpr static volatile uint8_t * const PORT() { return &PORTC; }
                constexpr static volatile uint8_t * const DDR() { return &DDRC; }
                constexpr static volatile uint8_t * const PIN() { return &PINC; }
            };

        public:

            using PortC = Port<PortCRegisters>;
#endif

#ifdef PORTD
        private:

            struct PortDRegisters
            {
                constexpr static volatile uint8_t * const PORT() { return &PORTD; }
                constexpr static volatile uint8_t * const DDR() { return &DDRD; }
                constexpr static volatile uint8_t * const PIN() { return &PIND; }
            };

        public:

            using PortD = Port<PortDRegisters>;
#endif

#ifdef PORTE
        private:

            struct PortERegisters
            {
                constexpr static volatile uint8_t * const PORT() { return &PORTE; }
                constexpr static volatile uint8_t * const DDR() { return &DDRE; }
                constexpr static volatile uint8_t * const PIN() { return &PINE; }
            };

        public:

            using PortE = Port<PortERegisters>;
#endif

        using Int0 = ExternalInterrupt<INT0_vect_num, PortD::Pin2, ISC01, ISC00, INT0, INTF0>;
        using Int1 = ExternalInterrupt<INT1_vect_num, PortD::Pin3, ISC11, ISC10, INT1, INTF1>;

        using Timer0 = Devices::Mcu::Timer0;
        using Timer2 = Devices::Mcu::Timer2;

        using PowerManagement = Devices::Mcu::PowerManagement;

        using UsartSpi = Devices::Mcu::UsartSpi;

        using Adc = Devices::Mcu::Adc<ClockFrequency>;

        using WatchDog = Devices::Mcu::WatchDog;
    };
}